package com.supermap.samplecode.realspace;

import java.awt.Color;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JDialog;
import com.supermap.data.CursorType;
import com.supermap.data.DatasetType;
import com.supermap.data.DatasetVector;
import com.supermap.data.DatasetVectorInfo;
import com.supermap.data.Datasource;
import com.supermap.data.GeoLine;
import com.supermap.data.GeoRegion;
import com.supermap.data.GeoStyle;
import com.supermap.data.GeoText;
import com.supermap.data.Geometrist;
import com.supermap.data.Geometry;
import com.supermap.data.Point2D;
import com.supermap.data.Point2Ds;
import com.supermap.data.Point3Ds;
import com.supermap.data.Recordset;
import com.supermap.data.TextAlignment;
import com.supermap.data.TextPart;
import com.supermap.data.TextStyle;
import com.supermap.data.Workspace;
import com.supermap.ui.MapControl;
import com.supermap.data.GeoLine3D;

/**
 * <p>
 * Title:剖面分析
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap iObjects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：如何基于地形和模型进行剖面分析
 * 2、示例数据：安装目录\SampleData\Chart\Chart.smwu;
 * 3、关键类型/成员: 
 *  	Workspace.open 方法
 *  	Workspace.getDatasources 方法
 *  	TerrainLayers.add 方法
 *  	Layers.add 方法
 *  	Scene.pixelToGlobe 方法
 *  	Scene.setCamera 方法
 *  	ProfileAnalyst.calculatePlumbProfile 方法
 *  	DatasetVector.getRecordset 方法
 *  	Recordset.BatchEditor.setMaxRecordCount 方法
 *  	Recordset.BatchEditor.begin 方法
 *  	Recordset.BatchEditor.update 方法
 *  	Datasets.create 方法
 *  	GeoLine.addPart 方法
 *  	GeoLine3D.addPart 方法
 *  	GeoLine.setStyle 方法
 *  	GeoLine3D.setStyle3D 方法
 *  	GeoText.setTextStyle 方法
 *  	TrackingLayer.add 方法
 *  	SceneControl.MouseEvent 事件
 * 4、使用步骤：
 *   (1) 运行应用程序
 *   (2) 待场景数据加载完，点击“绘制切割线”按钮，在场景中绘制切割线
 *   (3) 绘制切割线后，点击“分析”按钮，进行剖面分析。分析完成后弹出绘制有剖面图的窗体
 *   (4) 点击"重置相机"按钮，可以将场景定位到模型所在的范围
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 * Title: Profile analysis
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Copyright Statement----------------------------
 * SuperMap Objects Java Sample Code
 * Copyright: SuperMap Software Co., Ltd. All Rights Reserved.
 * ------------------------------------------------------------------------------
 *-----------------------Description--------------------------
 * 1, Introduction: Perform profile analysis based on terrain and model data
 * 2, Sample data: Installation directory\SampleData\BeijingDEM\BeijingDEM.smwu;
 * 3, Key class/method: 
 *  	Workspace.open method
 *  	Workspace.getDatasources method
 *  	TerrainLayers.add method
 *  	Layers.add method
 *  	Scene.pixelToGlobe method
 *  	Scene.setCamera method
 *  	ProfileAnalyst.calculatePlumbProfile method
 *  	DatasetVector.getRecordset method
 *  	Recordset.BatchEditor.setMaxRecordCount method
 *  	Recordset.BatchEditor.begin method
 *  	Recordset.BatchEditor.update method
 *  	Datasets.create method
 *  	GeoLine.addPart method
 *  	GeoLine3D.addPart method
 *  	GeoLine.setStyle method
 *  	GeoLine3D.setStyle3D method
 *  	GeoText.setTextStyle method
 *  	TrackingLayer.add method
 *  	SceneControl.MouseEvent event
 * 4, Step:
 *   (1) Run application
 *   (2) After the data is loaded, click "Draw line" to draw a line
 *   (3) After drawing, click "Analyze" to perform profile analysis
 *   (4) Click "Reset" to relocate the scene
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */

public class ResultDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private Workspace m_workspace;
	private MapControl m_mapControl;
	private Datasource m_datasource;

	// 剖面分析生成的三维剖面线数据集
	// The 3d result profile dataset built by profile anlaysis
	private DatasetVector m_resultProfile3D;

	// 切割线的起点和终点
	// Start and end point of the line
	private Point2D m_startPoint;
	private Point2D m_endPoint;

	// 使用数组存储所有三维剖面线对象
	// Use array to store all geoline 3d object
	private List<GeoLine3D> m_profileLine3Ds=new ArrayList<GeoLine3D>();

	// 地形剖面线在m_profileLine3Ds数组中的索引
	// Index of terrain line in m_profileLine3Ds array
	private int m_indexOfSurface = -1;

	// 规定绘制剖面线的区域长（X轴）为1000个单位，高（Y轴）为618个单位
	// The base width of line area(X axis) is 1000 unit while height is (Y axis) is 618
	private static final double m_baseWith = 1000.0;
	private  static final double m_baseHeight = 618.0;

	// 规定绘制剖面线的原点为（0,0）
	// Origin point of the line is (0,0)
	private Point2D m_origin = new Point2D(0, 0);
	
	// 最大高程和最小高程
	// Max and min elevation value
	private double m_maxElevation=Double.MIN_VALUE;
	private double m_minElevation=Double.MAX_VALUE;

	// 地球的长轴和扁率，用于计算测地线
	// Major axis and flatten ratio of earth
	private Double m_majorAxis;
	private Double m_flatten;

	/**
	 * 构造方法
	 * Construct method
	 */
	public ResultDialog(Workspace workspace, DatasetVector resultProfile3D, Point2D startPoint, Point2D endPoint, Datasource datasource)
	{
		try {
			this.setTitle(getText("title"));
			this.setSize(600, 400);
			this.setResizable(true);

			m_resultProfile3D = resultProfile3D;
			m_startPoint = startPoint;
			m_endPoint = endPoint;
			m_workspace = workspace;
			m_datasource = datasource;

			initialize();
			this.setVisible(true);
			this.m_mapControl.getMap().viewEntire();
			this.m_mapControl.getMap().refresh();
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}
	
	/**
	 * 未关闭窗体时再次分析时，重新加载
	 * Reload when analysis is performed again
	 */
	public void refresh(DatasetVector resultProfile3D,Point2D startPoint,Point2D endPoint)
	{
		m_resultProfile3D = resultProfile3D;
		m_startPoint = startPoint;
		m_endPoint = endPoint;
		initialize();
//		this.setVisible(true);
		this.m_mapControl.getMap().viewEntire();
		this.m_mapControl.getMap().refresh();
	}

	/**
	 * 初始化和计算
	 * Initialization
	 */
	private void initialize() {
		try	{
			// 添加地图控件到窗体
			// Add map control
			if (m_mapControl == null) {
				m_mapControl = new MapControl();
				this.add(m_mapControl);
			}
			m_mapControl.getMap().setWorkspace(m_workspace);

			// 获取所有剖面线，写入一个数组，并记录地形剖面线的索引
			// Get line and add them into a array
			m_profileLine3Ds = getProfileLine3D();

			// 创建用于存储剖面图的CAD数据集，获取记录集
			// Create CAD dataset used to store profile graph 
			String resultName = "ProfileResult";
			DatasetVector profileCAD = createProfileCAD(resultName);
			Recordset profileRecords = profileCAD.getRecordset(false, CursorType.DYNAMIC);

			// 获取长半径和扁率
			// Get Axis and flatten ratio
			m_majorAxis = m_resultProfile3D.getPrjCoordSys().getGeoCoordSys().getGeoDatum().getGeoSpheroid().getAxis();
			m_flatten = m_resultProfile3D.getPrjCoordSys().getGeoCoordSys().getGeoDatum().getGeoSpheroid().getFlatten();

			// 计算二维剖面图
			// Calculate 2D profile graph
			List<Geometry> geometris = calculateProfile();

			// 写入CAD数据集
			// Write CAD dataset
			draw(geometris, profileRecords);

			// 添加到地图控件中
			// Add them into map control
			m_mapControl.getMap().getLayers().clear();
			m_mapControl.getMap().getLayers().add(profileCAD, true);
			m_mapControl.getMap().setOverlapDisplayed(true);
			m_mapControl.getMap().viewEntire();
			m_mapControl.getMap().refresh();
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}
	
	/**
	 * 从结果剖面线数据集获取所有三维剖面线对象，并记录地形剖面线在数组中的索引
	 * Get all 3D line objects from the result 
	 * @return 所有三维剖面线对象数组
	 * @return all 3d line object array
	 */
	private List<GeoLine3D> getProfileLine3D()
	{
		try	{
			Recordset recordset = m_resultProfile3D.getRecordset(false, CursorType.STATIC);
			// 使用数组存放三维剖面线对象
			// Use array to store 3D profile line object
			List<GeoLine3D> profileLine3Ds = new ArrayList<GeoLine3D>();
			int index = -1;
			while (!recordset.isEOF())
			{
				index++;
				GeoLine3D line = (GeoLine3D)recordset.getGeometry();
				profileLine3Ds.add(line);
				Boolean isModel = recordset.getBoolean("IsModel");
				if (!isModel) {
					m_indexOfSurface = index;
				}
				recordset.moveNext();
			}
			recordset.dispose();
			return profileLine3Ds;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}
	
	/**
	 * 创建用于存储二维剖面图的CAD数据集
	 * Create CAD dataset to store 2D profile graph
	 * @param resultName CAD数据集的名称
	 * @param resultName Name of CAD dataset
	 * @return 新创建的数据集
	 * @return Dataset that has been created
	 */
	private DatasetVector createProfileCAD(String resultName)
	{
		try	{
			if (m_datasource.getDatasets().contains(resultName)) {
				m_datasource.getDatasets().delete(resultName);
			}
			DatasetVectorInfo cadInfo = new DatasetVectorInfo();
			cadInfo.setType(DatasetType.CAD);
			cadInfo.setName(resultName);
			DatasetVector profileCAD = m_datasource.getDatasets().create(cadInfo);
			return profileCAD;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}
	
	
	/**
	 * 计算二维剖面图，包括坐标轴和刻度、文本以及二维剖面线
	 * Calculate 2D profile graph, including axis, degree, text and line
	 * @return 二维剖面图的对象数组
	 * @return Array of 2D profile graph
	 */
	private List<Geometry> calculateProfile()
	{
		try
		{
			// 计算最大距离，实际为切割线的表面长度
			// Calculate maximum distance, the surface length of the line
			Double maxDistance = computeMaxDistance(m_startPoint, m_endPoint);

			// 计算剖面线中的最大高程和最小高程
			// Calculate maximum minimum elevation value
			computeElevation();

			List<Geometry> geometries =computeAllElements(maxDistance, m_maxElevation, m_minElevation);
			return geometries;
		}
		catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}

	/**
	 * 计算距离起点的最大距离，实际就是切割线的测地线长度
	 * Calculate the maximum distance of start point 
	 * @param startPoint 切割线的起点
	 * @param startPoint Start point of line
	 * @param endPoint 切割线的终点
	 * @param endPoint End point of line
	 * @return 最大距离
	 * @return Maximum distance
	 */
	private double computeMaxDistance(Point2D startPoint, Point2D endPoint)
	{
		try	{
			double maxDistance = Geometrist.computeGeodesicDistance(new Point2Ds(new Point2D[]{startPoint, endPoint}), m_majorAxis, m_flatten);
			return maxDistance;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return -1;
		}
	}
	
	/**
	 * 计算最大高程和最小高程
	 * Calculate maximum and minimum height value
	 */
	private void computeElevation()
	{
		try
		{
			for (int i = 0; i < m_profileLine3Ds.size(); i++)
			{
				GeoLine3D currentline = m_profileLine3Ds.get(i);
				int count = currentline.getPart(0).getCount();
				for (int j = 0; j < count; j++)
				{
					double currentElvation = currentline.getPart(0).getItem(j).getZ();
					
					if (currentElvation > m_maxElevation) {
						m_maxElevation = currentElvation;
					}
					if (currentElvation < m_minElevation) {
						m_minElevation = currentElvation;
					}
				}
			}
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}
	
	/**
	 * 计算剖面图的所有几何对象
	 * Calculate all geographic objects of profile graph
	 * @param maxDistance 距离起点的最大距离
	 * @param maxDistance Max distance from starting point
	 * @param maxElevation 最大高程
	 * @param maxElevation
	 * @param minElevation 最小高程
	 * @param minElevation
	 * @return 剖面图包含的几何对象的数组
	 * @return The array of the geographic objects of the profile graph
	 */
	private List<Geometry> computeAllElements(double maxDistance, double maxElevation, double minElevation)
	{
		try
		{
			// 使用数组存储所有对象，包括坐标轴及刻度、文本、剖面线对象
			// Use array to story all the objects
			List<Geometry> geometries = new ArrayList<Geometry>();

			NumberFormat formater=DecimalFormat.getInstance();
			formater.setMaximumFractionDigits(2);
			formater.setMinimumFractionDigits(2);
			
			// 绘制坐标轴
			// Draw coordinates axis
			// 计算X轴
			// Calculate X axis
			GeoStyle axisStyle = new GeoStyle();
			axisStyle.setLineSymbolID(963862);
			axisStyle.setLineColor(Color.BLACK);
			axisStyle.setLineWidth(0.1);
			GeoLine lineX = new GeoLine();
			Point2D endX = new Point2D(m_origin.getX() + (m_baseWith * 1.05), m_origin.getY());
			lineX.addPart(new Point2Ds(new Point2D[]{m_origin, endX}));
			lineX.setStyle(axisStyle);
			geometries.add(lineX);
			GeoText textX = createText(getText("textX"), 35, endX, TextAlignment.TOPLEFT, 15, -15);
			geometries.add(textX);
			double[] numsX = regulate(maxDistance, 0, 10);
			String value="";
			for (int i = 1; i <= numsX[0]; i++)	{
				GeoLine axisX = new GeoLine();
				double ratio = m_baseWith / maxDistance;
				Point2D start = new Point2D();
				// 将X轴最后一个刻度修改为最大距离
				// Modify the last scale of X axis to maximum distance
				if (numsX[i] > maxDistance)	{
					start.setX(m_baseWith);
					start.setY(m_origin.getY());
//					String temp=formater.format(maxDistance);
//					numsX[i] = Double.parseDouble(formater.format(maxDistance));
//					numsX[i] = Double.parseDouble(temp);
					numsX[i]=(Math.round(maxDistance*100))/100;
					value=Double.toString(numsX[i]);
				} else {
					start.setX(m_origin.getX() + numsX[i] * ratio);
					start.setY(m_origin.getY());
					value=Integer.toString((int)numsX[i]);
				}
				Point2D end = new Point2D(start.getX(), start.getY() - 20);
				axisX.addPart(new Point2Ds(new Point2D[]{start, end}));
				geometries.add(axisX);

				// 添加刻度文本
				// Add scale text
				GeoText text = createText(value, 35, end, TextAlignment.TOPCENTER, 0, -5);
				geometries.add(text);
			}

			// 计算Y轴
			// Calculate Y axis
			GeoLine lineY = new GeoLine();
			Point2D endY = new Point2D(m_origin.getX(), m_origin.getY() + m_baseHeight + m_baseWith* 0.05);
			lineY.addPart(new Point2Ds(new Point2D[]{m_origin, endY}));
			lineY.setStyle(axisStyle);
			geometries.add(lineY);
			GeoText textY = createText(getText("textY"), 35, endY, TextAlignment.MIDDLECENTER, 0, 30);
			geometries.add(textY);
			String elevationInfo = "Max(Z)=" + formater.format(maxElevation) + " " + "Min(Z)=" + formater.format(minElevation);
			GeoText elevationText = createText(elevationInfo, 30, endY, TextAlignment.MIDDLELEFT, 30, -15);
			geometries.add(elevationText);
			
			// 使Y轴绘制的高程范围占到Y轴总高度的三分之二
			// Make the height range draw by Y axis into 2/3 of overall Y axis height
			minElevation -= 2 * (maxElevation - minElevation);
			
			double[] numsY = regulate(maxElevation, minElevation, 10);
			minElevation = numsY[1];
			maxElevation = numsY[(int)numsY[0]];
			for (int i = 1; i <= numsY[0]; i++)	{
				GeoLine axisY = new GeoLine();
				double move = m_baseHeight / (numsY[0] - 1);
				Point2D start = new Point2D(m_origin.getX(), m_origin.getY() + move * (i - 1));
				Point2D end = new Point2D(start.getX() - 20, start.getY());
				axisY.addPart(new Point2Ds(new Point2D[]{start, end}));
				geometries.add(axisY);

				// 添加刻度文本
				// Add scale text
				GeoText text = createText(Integer.toString((int)numsY[i]), 35, end, TextAlignment.MIDDLERIGHT, -5, 0);
				geometries.add(text);
			}

			// 计算横轴和纵轴的绘制比率，即一个单位表示的长度
			// Calculate the draw ratio of X axis/Y axis
			double ratioX = maxDistance / m_baseWith;
			double ratioY = 0;
			ratioY = (maxElevation - minElevation) / m_baseHeight;
			// 计算要绘制的线，X轴方向为水平距离，Y轴方向为高程
			// Calculate the drawn line, X axis is horizontal, Y axis is vertical
			for (int i = 0; i < m_profileLine3Ds.size(); i++)
			{
				GeoLine3D profileline = m_profileLine3Ds.get(i);
				Point3Ds profilePoints = profileline.getPart(0);
				Point2Ds profilePoints2D = new Point2Ds();
				for (int j = 0; j < profilePoints.getCount(); j++) {
					Point2D currentPoint = new Point2D(profilePoints.getItem(j).getX(), profilePoints.getItem(j).getY());
					double distance = Geometrist.computeGeodesicDistance(new Point2Ds(new Point2D[] { m_startPoint, currentPoint }), m_majorAxis, m_flatten);

					Point2D point2D = new Point2D();
					point2D.setX(distance / ratioX);
					point2D.setY((profilePoints.getItem(j).getZ() - minElevation) / ratioY);
					profilePoints2D.add(point2D);
				}

				if (i == m_indexOfSurface) {
					// 将地形剖面线与地形部分组成面对象来绘制
					// Union terrain profile line and terrain to draw
					double max = Geometrist.computeGeodesicDistance(new Point2Ds(new Point2D[] { m_startPoint, m_endPoint }), m_majorAxis, m_flatten);
					profilePoints2D.add(new Point2D(m_origin.getX() + max / ratioX, m_origin.getY()));
					profilePoints2D.add(m_origin);
					GeoRegion geoRegion = new GeoRegion();
					geoRegion.addPart(profilePoints2D);
					GeoStyle style = new GeoStyle();
					style.setFillForeColor(Color.BLUE);
					style.setFillSymbolID(2);
					style.setLineWidth(0.1);
					geoRegion.setStyle(style);
					geometries.add(geoRegion);
				} else {
					GeoLine geoLine = new GeoLine();
					geoLine.addPart(profilePoints2D);
					geometries.add(geoLine);
				}
			}
			return geometries;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}
	
	/**
	 * 根据指定的参数创建文本对象
	 * Create text object according to the specified parameters
	 * @param text 文本内容
	 * @param fontHeight 文本高度
	 * @param reference 文本位置的参考点
	 * @param offsetX X方向偏移量
	 * @param offsetY Y方向偏移量
	 */
	private GeoText createText(String text, int fontHeight, Point2D reference, TextAlignment alignment, int offsetX, int offsetY)
	{
		try	{
			GeoText geoText = new GeoText();
			TextPart part = new TextPart();
			part.setText(text);
			part.setX(reference.getX()+ offsetX);
			part.setY(reference.getY() + offsetY);
			geoText.addPart(part);
			TextStyle textStyle = new TextStyle();
			textStyle.setFontHeight(fontHeight);
			textStyle.setSizeFixed(false);
			textStyle.setAlignment(alignment);
			geoText.setTextStyle(textStyle);
			return geoText;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}
	
	/**
	 * 根据最大最小值计算坐标轴步长和刻度值
	 * Calculate axis step length and scale value according to the max/min value
	 * @param max 最大值
	 * @param min 最小值
	 * @param maxAxisNum 最大刻度数
	 * @return 刻度值数组，其中第一个元素为标识刻度数目的值
	 * @return Scale array, the first element is the value to identify scale number
	 */
	private double[] regulate(double max, double min, int maxAxisNum)
	{
		try	{
			double[] nums = new double[maxAxisNum + 2];
			if (maxAxisNum < 1 || max < min) {
				return null;
			}
			double delta = max - min;
			if (delta < 1.0) {
				max += (1.0 - delta) / 2.0;
				min -= (1.0 - delta) / 2.0;
			}
			delta = max - min;

			int exp = (int)(Math.log(delta) / Math.log(10)) - 2;
			double multiplier = Math.pow(10, exp);
			double[] solutions = new double[] { 1, 2, 2.5, 5, 10, 20, 25, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500 };
			int i;
			for (i = 0; i < solutions.length; i++) {
				double multiCal = multiplier * solutions[i];
				if (((int)(delta / multiCal) + 1) <= maxAxisNum) {
					break;
				}
			}

			double interval = multiplier * solutions[i];
			double startPoint;
			if (min == 0) {
				startPoint = 0;
			} else {
				startPoint = (Math.ceil(min / interval) - 1) * interval;
			}
			int axisIndex;
			for (axisIndex = 1; axisIndex <= maxAxisNum + 1; axisIndex++) {
				nums[axisIndex] = (startPoint + interval * (axisIndex - 1));
				if (startPoint + interval * (axisIndex - 1) > max) {
					break;
				}
			}
			nums[0] = axisIndex;
			return nums;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
			return null;
		}
	}
	
	/**
	 * 将剖面图的所有对象添加到结果数据集中
	 * Add all objects of profile graph into result datasets
	 * @param geometries 剖面图的所有几何对象
	 * @param geometries All geometry objects in profile graph
	 * @param profileRecords 结果数据集的记录集
	 * @param profileRecords Records in result datasets
	 */
	private void draw(List<Geometry> geometries, Recordset profileRecords)
	{
		try	{
			// 使用批量添加
			// Batch add
			Recordset.BatchEditor editor = profileRecords.getBatch();
			editor.setMaxRecordCount(5);
			editor.begin();
			for(Geometry geometry : geometries)	{
				profileRecords.addNew(geometry);
			}
			editor.update();
			profileRecords.close();
			profileRecords.dispose();
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}
	
	private String getText(String name)
	{
		String text = "";
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			if(name == "title")
			{
				text = "剖面图";
			}
			if(name == "textX")
			{
				text = "D(距离,米)";
			}
			if(name == "textY")
			{
				text = "Z(高程,米)";
			}
		}
		else
		{
			if(name == "title")
			{
				text = "Profile graph";
			}
			if(name == "textX")
			{
				text = "D(Distance, meter)";
			}
			if(name == "textY")
			{
				text = "Z(Height, meter)";
			}
		}
		return text;
	}
}

